(function() {
    'use strict';

    angular.module('wemSettingsApp', ['wemApp']);
})();

(function() {
    'use strict';

    angular.module('wemSettingsApp').controller('ManageWemSettingsController', ManageWemSettingsController);

    ManageWemSettingsController.$inject = ['i18nService', 'notificationService', 'modalService', 'loadingSpinnerService', 'wemSettingsService'];

    function ManageWemSettingsController(i18nService, notificationService, modalService, loadingSpinnerService, wemSettingsService) {
        var vm = this;
        vm.hasSettings = ManagersContext.contextServerSetup;

        vm.saveSettings = saveSettings;
        vm.deleteSettings = deleteSettings;
        vm.tryPurgeScope = tryPurgeScope;
        vm.getStatusColor = getStatusColor;
        vm.seeErrorDetails = seeErrorDetails;

        _init();

        function saveSettings() {
            loadingSpinnerService.show();
            if (vm.settingsForm.$invalid) {
                console.log("invalid settings");
                vm.settingsForm.showErrors = true;

                var parsedErrors = ManagersUtils.parseErrorsInForm(vm.settingsForm);
                if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                    loadingSpinnerService.hide();
                    ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                    return;
                }
            }

            wemSettingsService.saveSettings(vm.settings)
                .then(function (response) {
                    loadingSpinnerService.hide();
                    if (response.data.error == undefined) {
                        vm.hasSettings = true;
                        vm.settings = response.data;
                        notificationService.successToast(i18nService.message('serverSettings.manageSettings.data.save.ok'));
                    } else {
                        wemSettingsService.handleErrors(i18nService.message('serverSettings.manageSettings.save.error'), response.data, vm.settings.contextServerURL, vm.settings.contextServerUsername);
                    }
                }, function (response) {
                    loadingSpinnerService.hide();
                    wemSettingsService.handleErrors(i18nService.message('serverSettings.manageSettings.save.error'), response.data, vm.settings.contextServerURL, vm.settings.contextServerUsername);
                });
        }

        function deleteSettings() {
            loadingSpinnerService.show();
            wemSettingsService.deleteSettings()
                .then(function (response) {
                    loadingSpinnerService.hide();
                    vm.hasSettings = false;
                    vm.settings = response.data;
                    _loadDefaultSettings();
                    notificationService.successToast(i18nService.message('serverSettings.manageSettings.delete.ok'));
                },
                function(response) {
                    loadingSpinnerService.hide();
                    wemSettingsService.handleErrors(i18nService.message('serverSettings.manageSettings.delete.error'), response.data, vm.settings.contextServerURL, vm.settings.contextServerUsername);
                });
        }

        function tryPurgeScope() {
            // ask confirmation
            var contextServerScope = vm.settings.contextServerScope;
            modalService.newAlertModal({
                title: i18nService.format("serverSettings.manageSettings.purge.scope.confirm", contextServerScope),
                bodyMessage: i18nService.format("serverSettings.manageSettings.purge.scope.message", contextServerScope)
            }, function () {
                loadingSpinnerService.show();
                wemSettingsService.tryPurgeScope(contextServerScope)
                    .then(function (response) {
                        loadingSpinnerService.hide();
                        notificationService.successToast(i18nService.message('serverSettings.manageSettings.purge.scope.ok'));
                    }, function (response) {
                        loadingSpinnerService.hide();
                        wemSettingsService.handleErrors(i18nService.message('serverSettings.manageSettings.purge.scope.error'), response.data, vm.settings.contextServerURL, vm.settings.contextServerUsername);
                    });

            });
        }

        function getStatusColor(currentServerStatus) {
            if (currentServerStatus == 'ONLINE') {
                return '#4CAF50';
            } else if (currentServerStatus == 'UNKNOWN') {
                return '#FF9800';
            } else {
                return '#F44336';
            }
        }

        function seeErrorDetails(contextServerError) {
            var throwable = '\n';
            angular.forEach(contextServerError.throwable, function(v) {
                throwable += v + '\n';
            });

            modalService.newInfoModal({
                title: i18nService.message('wem.label.error') + ' HTTP ' + contextServerError.httpStatus + ' ' + i18nService.message('wem.label.on') + ' ' + moment(contextServerError.timestamp).format('LLL'),
                bodyMessage: contextServerError.message + throwable,
                size: 'lg',
                windowClass: 'wem-settings-error-modal'
            });
        }

        // Private function under this line
        function _init() {
            wemSettingsService.getSettings()
                .then(function (response) {
                    vm.settings = response.data;
                    if (!vm.hasSettings) {
                        _loadDefaultSettings();
                    }
                }, function () {
                    notificationService.errorToast(i18nService.message('serverSettings.manageSettings.load.error'));
                });
        }

        function _loadDefaultSettings() {
            vm.settings.contextServerURL = 'https://localhost:9443';
            vm.settings.contextServerScope = ManagersContext.currentSiteKey;
            vm.settings.contextServerTrustAllCertificates = true;
            vm.settings.contextServerUsername = 'karaf';
            vm.settings.contextServerPassword = 'karaf';
            vm.settings.timeoutInMilliseconds = 1500;
        }
    }
})();
(function() {
    'use strict';

    angular.module('wemSettingsApp').service('wemSettingsService', wemSettingsService);

    wemSettingsService.$inject = ['$http', 'i18nService', 'notificationService'];

    function wemSettingsService($http, i18nService, notificationService) {
        var _settingsUrlAction = ManagersContext.basePreview + ManagersContext.currentSiteHomePagePath + '.wemSettings.do';

        return {
            getSettings: getSettings,
            saveSettings: saveSettings,
            deleteSettings: deleteSettings,
            tryPurgeScope: tryPurgeScope,
            handleErrors: handleErrors
        };

        function getSettings() {
            return $http.get(_settingsUrlAction);
        }

        function saveSettings(settings) {
            return $http.post(_settingsUrlAction, settings);
        }

        function deleteSettings() {
            return $http.post(ManagersContext.basePreview + ManagersContext.currentSiteHomePagePath + '.removeWemSettings.do');
        }

        function tryPurgeScope(contextServerScope) {
            return $http.delete(ManagersContext.proxyServletUrl + '/cxs/cluster/' + contextServerScope);
        }

        function handleErrors(errorMsg, data, contextServerURL, contextServerUsername) {
            var exceptionToKey = {
                UnknownHostException: {
                    key: 'serverSettings.manageSettings.save.error.unknownHost',
                    param: contextServerURL
                },
                '401': {
                    key: 'serverSettings.manageSettings.save.error.unauthorized',
                    param: contextServerUsername ? contextServerUsername : '__void__'
                },
                SSLHandshakeException: {
                    key: 'serverSettings.manageSettings.save.error.ssl',
                    param: undefined
                }
            };

            // check if we do special handling of the returned exception
            var detail = exceptionToKey[data.type];
            var detailMsg;
            if (detail) {
                detailMsg = ': ' + i18nService.format(detail.key, detail.param);
            } else {
                // if we have an error append it to the default error message
                detailMsg = (data.error ? ': ' + data.error : '');
            }
            errorMsg = errorMsg + detailMsg;
            notificationService.errorToast(errorMsg);
        }
    }
})();