(function() {
    'use strict';

    angular.module('wemApp.services').service('referrerService', referrerService);

    referrerService.$inject = ['$q', 'i18nService', 'unomiQueryService', 'unomiProfileService'];

    function referrerService($q, i18nService, unomiQueryService, unomiProfileService) {
        var _selectedQuery;
        var _selectedReferrer;
        var _scopeCondition = {
            "type": "sessionPropertyCondition",
            "parameterValues": {
                "propertyName": "scope",
                "comparisonOperator": "equals",
                "propertyValue": ManagersContext.dmfScope
            }
        };

        var _viewFromReferrerEventCondition = {
            "type": "eventTypeCondition",
            "parameterValues": {
                "eventTypeId": "viewFromReferrer"
            }
        };

        return {
            getReferrers: getReferrers,
            getReferrerDetails: getReferrerDetails,
            getReferrerQueries: getReferrerQueries,
            getQueries: getQueries,
            getQueryDetails: getQueryDetails,
            getQueryReferrers: getQueryReferrers,
            setSelectedReferrer: setSelectedReferrer,
            setSelectedQuery: setSelectedQuery,
            decodeQuery: decodeQuery
        };

        function getReferrers(from, to) {
            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    _viewFromReferrerEventCondition)
            };

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerHost', referrerEventsByDateCondition)
                .then(function (response) {
                    var data = response.data;
                    var referrers = [];
                    for (var referrer in data) {
                        if (ManagersUtils.acceptProperty(data, referrer)) {
                            var occurrences = data[referrer];
                            referrers.push({
                                host: referrer,
                                occurrences: occurrences,
                                importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                            });
                        }
                    }

                    var result = {data: referrers};
                    deferred.resolve(result);
                },function () {
                    deferred.reject();
                });

            return deferred.promise;
        }

        function getReferrerDetails(host, offset, length, order) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || _selectedReferrer;

            var condition = {
                "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                    "type": "sessionPropertyCondition",
                    "parameterValues": {
                        "propertyName": "properties.referrerHost",
                        "comparisonOperator": "equals",
                        "propertyValue": host
                    }
                }),
                "limit": length,
                "offset": offset,
                "sortby": order,
                "text": null
            };

            return unomiProfileService.searchSessions(condition, 'profiles');
        }

        function getReferrerQueries(host) {
            // if we didn't provide a host, use the one potentially already selected
            host = host || _selectedReferrer;

            var referrerQueries = new ManagersUtils.AggregateDisplay(host, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(_scopeCondition, _viewFromReferrerEventCondition)};
            condition.condition.parameterValues.subConditions.push({
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerHost",
                    "comparisonOperator": "equals",
                    "propertyValue": host
                }
            });

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerQuery', condition).then(function (response) {
                referrerQueries.setDetails('queries', response.data);
            });

            return referrerQueries;
        }

        function getQueries(from, to) {
            var deferred = $q.defer();
            var referrerEventsByDateCondition = {
                "condition": ManagersUtils.booleanAnd(ManagersUtils.getDateRangeConditionBody(from, to),
                    _viewFromReferrerEventCondition)
            };

            var eventTypeCondition = angular.copy(referrerEventsByDateCondition.condition.parameterValues.subConditions[1]);
            referrerEventsByDateCondition.condition.parameterValues.subConditions[1] = ManagersUtils.booleanAnd(eventTypeCondition, {
                "type": "eventPropertyCondition",
                "parameterValues": {
                    "propertyName": "target.properties.pageInfo.referrerQuery",
                    "comparisonOperator": "exists"
                }
            });

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerQuery', referrerEventsByDateCondition)
                .then(function (response) {
                    var data = response.data;
                    var queryList = [];
                    for (var query in data) {
                        if (ManagersUtils.acceptProperty(data, query)) {
                            var occurrences = data[query];
                            queryList.push({
                                referrerQuery: query,
                                occurrences: occurrences,
                                importance: ManagersUtils.getPercentage(occurrences / data._filtered)
                            });
                        }
                    }

                    var result = {data: queryList};
                    deferred.resolve(result);
                },function () {
                    deferred.reject();
                });

            return deferred.promise;
        }

        function getQueryDetails(query, offset, length, order) {
            query = decodeQuery(query) || _selectedQuery;

            var condition;
            if ('_missing' === query) {
                condition = {
                    "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "missing"
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            } else {
                condition = {
                    "condition": ManagersUtils.booleanAnd(_scopeCondition, {
                        "type": "sessionPropertyCondition",
                        "parameterValues": {
                            "propertyName": "properties.referrerQuery",
                            "comparisonOperator": "equals",
                            "propertyValue": query
                        }
                    }),
                    "limit": length,
                    "offset": offset,
                    "sortby": order,
                    "text": null
                };
            }

            return unomiProfileService.searchSessions(condition, 'profiles');
        }

        function getQueryReferrers(query) {
            // if we didn't provide a search term, use the one potentially already selected
            query = decodeQuery(query) || _selectedQuery;

            var incomingSearch = new ManagersUtils.AggregateDisplay(query, -1, i18nService);

            var condition = {"condition": ManagersUtils.booleanAnd(_scopeCondition, _viewFromReferrerEventCondition)};
            if ('_missing' === query) {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "missing"
                    }
                });
            } else {
                condition.condition.parameterValues.subConditions.push({
                    "type": "eventPropertyCondition",
                    "parameterValues": {
                        "propertyName": "target.properties.pageInfo.referrerQuery",
                        "comparisonOperator": "equals",
                        "propertyValue": query
                    }
                });
            }

            unomiQueryService.query('event', 'target.properties.pageInfo.referrerHost', condition).then(function (response) {
                incomingSearch.setDetails('hosts', response.data);
            });

            incomingSearch.incomingSearchString = ManagersUtils.humanReadableMissingIfNeeded(query, i18nService);

            return incomingSearch;
        }

        function setSelectedReferrer(referrer) {
            _selectedReferrer = referrer;
        }

        function setSelectedQuery(query) {
            _selectedQuery = query;
            return _encodeQuery(query);
        }

        function decodeQuery(query) {
            return query.replace(/\+/g, ' ')
        }

        // Private functions under this line
        function _encodeQuery(query) {
            return query.replace(/ /g, '+')
        }
    }
})();